/*
 *  myqq.c
 *
 *  main() A small console for MyQQ.
 *
 *  Copyright (C) 2008  Huang Guan (gdxxhg@gmail.com)
 *
 *  2008-01-31 Created.
 *  2008-2-5   Add more commands.
 *  2008-7-15  Mr. Wugui said "There's no accident in the world.", 
 *	       but I always see accident in my code :)
 *
 *  Description: This file mainly includes the functions about 
 *               Parsing Input Commands.
 *               
 */


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#ifdef __WIN32__
#include <winsock.h>
#include <wininet.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif
#include "qqclient.h"
#include "buddy.h"
#include "qun.h"
#include "group.h"
#include "memory.h"
#include "utf8.h"
#include "config.h"
#include "qqsocket.h"


#define MSG need_reset = 1; printf

#define QUN_BUF_SIZE	80*100
#define BUDDY_BUF_SIZE	80*500
#define PRINT_BUF_SIZE	80*500*3


static char* qun_buf, *buddy_buf, *print_buf;
static uint to_uid = 0;		//send message to that guy.
static uint qun_int_uid;		//internal qun id if entered.
static char input[1024];
static int enter = 0;
static qqclient* qq;
static int need_reset;
enum{
	CMD_EXIT = 0, CMD_EXIT2,
	CMD_SAY, CMD_SAY2,
	CMD_TO, CMD_TO2,
	CMD_HELP,
	CMD_STATUS,
	CMD_ENTER, CMD_ENTER2,
	CMD_LEAVE, CMD_LEAVE2,
	CMD_WHO, CMD_WHO2,
	CMD_VIEW, CMD_VIEW2,
	CMD_QUN, CMD_QUN2,
	CMD_INFO, CMD_INFO2,
	CMD_UPDATE, CMD_UPDATE2,
	CMD_CHANGE, CMD_CHANGE2,
	CMD_TEST,
	CMD_VERIFY, CMD_VERIFY2
};
static char commands[][16]={
	"exit", "x",
	"say", "s",
	"to", "t",
	"help",
	"status",
	"enter", "e",
	"leave", "l",
	"who", "w",
	"view", "v",
	"qun", "q",
	"info", "i",
	"update", "u",
	"change", "c",
	"test",
	"verify", "r"
};

static char help_msg[]=
"Welcome to use MyQQ2008 alpha\n"
"This is a tiny console IM, made for programmers and computer fans. Enjoy it!\n"
"help: Show this message.\n"
"to/t [UID|N]: Say something to UID:NUMBER.\n"
"say/s MESSAGE: Send message to who you are talking to.\n"
"status [online|away|busy|killme|hidden]: Change status.\n"
"who/w: List all online buddies. List online Qun members if you are in a Qun.\n"
"view/v: List all buddies. List all Qun members if you are in a Qun.\n"
"enter/e [GID|N]: Enter a Qun, and you can talk to members by command 'say'\n"
"leave/l: Leave a Qun.\n"
"qun/q: Show all QQ Quns.\n"
"info/i: Show the information of a buddy or a Qun. Use 'to' or 'enter' fisrt.\n"
"update/u: Update the buddy list and the Qun list.\n"
"change/c: Change ID to login.\n"
"verify/r: Input the verification code.\n"
"exit/x: Logout.\n"
;


#define RESET_INPUT \
	if( need_reset ){\
	if( enter ){ \
		MSG("In {%s}> ", to_utf8( myqq_get_qun_name( qq, qun_int_uid ) ) ); \
	}else{ \
		MSG("To [%s]> ", to_utf8( myqq_get_buddy_name( qq, to_uid ) ) );} \
	fflush( stdout ); \
	need_reset = 0;}




static char* to_gb( char* s )
{
#ifdef __WIN32__
	return s;
#else
	memset( print_buf, 0, PRINT_BUF_SIZE );
	utf8_to_gb( s, print_buf, PRINT_BUF_SIZE-1 );
	return print_buf;
#endif
}
static char* to_utf8( char* s )
{
#ifdef __WIN32__
	return s;
#else
	memset( print_buf, 0, PRINT_BUF_SIZE );
	gb_to_utf8( s, print_buf, PRINT_BUF_SIZE-1 );
	return print_buf;
#endif
}

static int getline(char *s, int lim) {
    char *t;
    int c;

    t = s;
    while (--lim>1 && (c=getchar()) != EOF && c != '\n')
        *s++ = c;
    *s = '\0';
    return s - t;
}

static char* skip_line( char* p, int ln )
{
	while( *p && ln-- )
	{
		p ++;
		while( *p && *p!='\n' ) p++;
	}
	return p;
}
static char* myqq_get_buddy_name( qqclient* qq, uint uid )
{
	static char tmp[16];
	qqbuddy* b = buddy_get( qq, uid, 0 );
	if( b )
		return b->nickname;
	if( uid == 10000 )
		return "System";
	if( uid != 0 ){
		sprintf( tmp, "%u" , uid );
		return tmp;
	}
	return "Nobody";
}
static char* myqq_get_qun_name( qqclient* qq, uint uid )
{
	static char tmp[16];
	qqqun* q = qun_get( qq, uid, 0 );
	if( q )
		return q->name;
	if( uid != 0 ){
		sprintf( tmp, "%u" , uid );
		return tmp;
	}
	return "[q==NULL]";
}
static char* myqq_get_qun_member_name( qqclient* qq, uint int_uid, uint uid )
{
	static char tmp[16];
	qqqun* q = qun_get( qq, int_uid, 0 );
	if( q ){
		qunmember* m = qun_member_get( qq, q, uid, 0 );
		if( m )
			return m->nickname;
		if( uid != 0 ){
			sprintf( tmp, "%u" , uid );
			return tmp;
		}
		return "[m==NULL]";
	}
	return "[q==NULL]";
}
static int myqq_send_im_to_qun( qqclient* qq, uint int_uid, char* msg, int wait )
{
	qun_send_message( qq, int_uid, msg );
	if( wait )
	{
		if( qqclient_wait( qq, 15 ) < 0 )
			return -1;
	}
	return 0;
}
static int myqq_send_im_to_buddy( qqclient* qq, uint int_uid, char* msg, int wait )
{
	buddy_send_message( qq, int_uid, msg );
	if( wait )
	{
		if( qqclient_wait( qq, 15 ) < 0 )
			return -1;
	}
	return 0;
}

static int myqq_get_buddy_info( qqclient* qq, uint uid, char* buf, int size )
{
	qqbuddy *b = buddy_get( qq, uid, 0 );
	if( size < 256 )
		return -1;
	if( b == NULL )
		return -2;
	int len, ip = htonl(b->ip);
	len = sprintf( buf,     "BuddyName           %-32s\n"
				"UserID              %-32d\n"
				"IPAddress           %-32s\n"
				"Port                %-32d\n"
				"Face                %-32d\n"
				"Age                 %-32d\n"
				"Sex                 %-32s\n"
				"Mode                %-32s\n",
		b->nickname, b->number, inet_ntoa( *(struct in_addr*)&ip ), b->port, b->face, b->age,
		(b->sex==0x00)?"Male": (b->sex==0x01)?"Female":"Asexual",
		(b->status==QQ_OFFLINE)?"OffLine": (b->status==QQ_AWAY)?"Away" : "Online" );
	return len;
}


//Note: this function change the source string directly.
static char* util_escape( char* str )
{
	unsigned char* ch;
	ch = (unsigned char*)str;
	while( *ch )
	{
		if( *ch < 0x80 ) //ASCII??
		{
			if( !isprint(*ch) ) //if not printable??
				*ch = ' ';	//use space instead..
			ch ++;	//skip one
		}else{
			ch +=2; //skip two
		}
	}
	return str;
}

static char* mode_string( int mode )
{
	switch( mode ){
	case QQ_ONLINE:
		return "Online";
	case QQ_BUSY:
		return "Busy";
	case QQ_OFFLINE:
		return "Offline";
	case QQ_HIDDEN:
		return "Hidden";
	case QQ_KILLME:
		return "Kill Me";
	case QQ_QUIET:
		return "Quiet";
	}
	return "Unknown";
}
/*   The output buf looks like this:
L8bit  L16bit        L16bit        L32bit
1      357339036     online		Xiao xia
2      273310179     offline	Huang Guan
*/
//Note: size must be big enough
static int myqq_get_buddy_list( qqclient* qq, char* buf, int size, char online )
{
	int i, len = 0;
	//avoid editing the array
	buf[0] = 0;
	pthread_mutex_lock( &qq->buddy_list.mutex );
	int ln = 1;
	for( i=0; i<qq->buddy_list.count; i++ )
	{
		qqbuddy * b = (qqbuddy*)qq->buddy_list.items[i];
		if( online && b->status == QQ_OFFLINE ) continue;
		len = sprintf( buf, "%s%-8d%-16d%-16s%-16.64s\n", buf, ln ++, b->number, 
			mode_string( (int) b->status ), util_escape( b->nickname ) );
		if( len + 80 > size ) break;
	}
	pthread_mutex_unlock( &qq->buddy_list.mutex );
	return len;
}

/*   The output buf looks like this:
L8bit  L16bit        L16bit        L32bit
1      467788923     12118724	Xinxing Experimental School
2      489234223     13223423	SGOS2007
*/
//Note: size must be big enough
static int myqq_get_qun_list( qqclient* qq, char* buf, int size )
{
	int i, len = 0, ln=1;
	//avoid editing the array
	buf[0] = 0;
	pthread_mutex_lock( &qq->qun_list.mutex );
	for( i=0; i<qq->qun_list.count; i++ )
	{
		qqqun * q = (qqqun *)qq->qun_list.items[i];
		len = sprintf( buf, "%s%-8d%-16d%-16d%-16.64s\n", buf, ln ++, q->number, 
			q->ext_number, util_escape( q->name ) );
		if( len + 80 > size ) break;
	}
	pthread_mutex_unlock( &qq->qun_list.mutex );
	return len;
}

/*   The output buf looks like this:
L8bit  L16bit        L16bit        L32bit
1      357339036     Manager	Xiaoxia
2      273310179     Fellow		Huang Guan
*/
//Note: size must be big enough
static int myqq_get_qun_member_list( qqclient* qq, uint int_uid, char* buf, int size, char online )
{
	qqqun * q = qun_get( qq, int_uid, 0 );
	if( !q )return 0;
	//Hope the Qun won't get removed while we are using it. 
	int i, len = 0, ln = 1;
	buf[0] = 0;
	pthread_mutex_lock( &q->member_list.mutex );
	for( i=0; i<q->member_list.count; i++ )
	{
		qunmember * m = (qunmember *)q->member_list.items[i];
		if( online && m->status == QQ_OFFLINE ) continue;
		len = sprintf( buf, "%s%-8d%-16d%-16s%-16.64s\n", buf, ln++, m->number, 
			(m->role&1)?"Admin":"Fellow", util_escape( m->nickname ) );
		if( len + 80 > size )
			break;
	}
	pthread_mutex_unlock( &q->member_list.mutex );
	return len;
}

//interface for getting qun information
/* Output style:
*/
static int myqq_get_qun_info( qqclient* qq, uint int_uid, char* buf, int size )
{
	char cate_str[4][10] = {"Classmate", "Friend", "Workmate", "Other" };
	qqqun *q = qun_get( qq, int_uid, 0 );
	if( !q )return 0;
	int len;
	if( size < 256 )
		return -1;

	if( q == NULL )
		return -2;
	len = sprintf( buf, 	"Name                %-32s\n"
				"InternalID          %-32d\n"
				"ExternalID          %-32d\n"
				"Type                %-32s\n"
				"AuthenMethod        %-32s\n"
				"Category            %-32s\n"
				"Owner               %-32d\n"
				"Members             %-32d\n"
				"Introduction        \n%-32s\n"
				"Announcement        \n%-32s\n",
		q->name, q->number, q->ext_number, (q->type==0x01)?"Normal":"Special",
		(q->auth_type==0x01)?"No": (q->auth_type==0x02)?"Yes":
			(q->auth_type==0x03)?"RejectAll":"Unknown",
		q->category > 3 ? cate_str[3] : cate_str[(int)q->category],
		q->owner, q->member_list.count, q->intro, q->ann );
	return len;
}


void buddy_msg_callback ( qqclient* qq, uint uid, time_t t, char* msg )
{
	char timestr[12];
	char msgstr[64];
	struct tm * timeinfo;
	char* nick = myqq_get_buddy_name( qq, uid );
  	timeinfo = localtime ( &t );
	strftime( timestr, 12, "%H:%M:%S", timeinfo );
	char tmp[20], *p;
	sprintf( tmp, "%-16d", uid );
	p = strstr( buddy_buf, tmp );
	if( p )
	{
		p -= 8;
		if( p>=buddy_buf )
		{
			int ln;
			sscanf( p, "%d", &ln );
			sprintf( msgstr, "\n%d)%s[", ln, timestr );
			strcat( msgstr, nick );
			strcat( msgstr, "] " );
			MSG( to_utf8( msgstr ) );
			puts( to_utf8( msg ) );
			RESET_INPUT
		}
	}else{
		sprintf( msgstr, "\n%s[", timestr );
		strcat( msgstr, nick );
		strcat( msgstr, "] " );
		MSG( to_utf8( msgstr ) );
		puts( to_utf8( msg ) );
		RESET_INPUT
	}
}

void qun_msg_callback ( qqclient* qq, uint uid, uint int_uid,
	time_t t, char* msg )
{
	char timestr[12];
	char msgstr[64];
	char* qun_name = myqq_get_qun_name( qq, int_uid );
	char* nick = myqq_get_qun_member_name( qq, int_uid, uid );
	struct tm * timeinfo;
  	timeinfo = localtime ( &t );
	strftime( timestr, 12, "%H:%M:%S", timeinfo );
	char tmp[20], *p;
	sprintf( tmp, "%-16d", int_uid );
	p = strstr( qun_buf, tmp );
	if( p )
	{
		p -= 8;
		if( p>=qun_buf )
		{
			int ln;
			sscanf( p, "%d", &ln );
			sprintf( msgstr, "\n%d)%s{", ln, timestr );
			strcat( msgstr, qun_name );
			strcat( msgstr, "}[" );
			strcat( msgstr, nick );
			strcat( msgstr, "] " );
			MSG( to_utf8( msgstr ) );
			puts( to_utf8( msg ) );
			RESET_INPUT
		}
	}else{
		sprintf( msgstr, "\n%s{", timestr );
		strcat( msgstr, qun_name );
		strcat( msgstr, "}[" );
		strcat( msgstr, nick );
		strcat( msgstr, "] " );
		MSG( to_utf8( msgstr ) );
		puts( to_utf8( msg ) );
		RESET_INPUT
	}
}

int main(int argc, char** argv)
{
	int cmdid, lastcmd=-1, len;
	char cmd[16], arg[1008];
	srand(time(NULL));
	//init data
	config_init();
	qqsocket_init();
	NEW( qq, sizeof(qqclient) );
	//login
	if( argc<3 )
	{
		uint uid;
		char password[32];
		MSG("Please input your QQ ID: ");
		scanf("%u", &uid );
		MSG("Please input you Password: ");
		scanf("%s", password );
		MSG("Login in Hidden Mode? (y/n) ");
		scanf("%s", input );
		qqclient_create( qq, uid, password );
		qq->mode = *input=='y' ? QQ_HIDDEN : QQ_ONLINE;
		qqclient_login( qq );
		scanf("%c", input ); //If I don't add this, it will display '>' twice.
	}else{
		qqclient_create( qq, atoi(argv[1]), argv[2] );
		if( argc > 3 )
			qq->mode = atoi(argv[3])!=0 ? QQ_HIDDEN : QQ_ONLINE;
		qqclient_login( qq );
	}
	MSG("Logging...\n");
	while( qq->process == P_LOGGING )
		qqclient_wait( qq, 1 );
	while( qq->process == P_VERIFYING ){
		MSG("Please input the verification code: ");
		scanf( "%s", input );
		qqclient_verify( qq, *(uint*)input );
		while( qq->process == P_LOGGING )
			qqclient_wait( qq, 1 );
	}
	if( qq->process != P_LOGIN ){
		switch( qq->process ){
		case P_ERROR:
			MSG("Network Error\n");
			break;
		case P_DENIED:
			MSG("Your QQ number is denied to login. See http://jihuo.qq.com\n");
			break;
		case P_WRONGPASS:
			MSG("Your password is incorrect.\n");
			break;
		}
		qqclient_logout( qq );
		qqclient_cleanup( qq );
		return 0;
	}
	MSG( help_msg );
	NEW( qun_buf, QUN_BUF_SIZE );
	NEW( buddy_buf, BUDDY_BUF_SIZE );
	NEW( print_buf, PRINT_BUF_SIZE );
	while( qq->process != P_INIT ){
		RESET_INPUT
		len = getline( input, 1023 );
		if( len < 1 ) continue;
		char* sp = strchr( input, ' ' );
		if( sp ){
			*sp = '\0';
			strncpy( cmd, input, 16-1 );
			strncpy( arg, sp+1, 1008-1 );
			*sp = ' ';
		}else{
			strncpy( cmd, input, 16-1 );
			arg[0] = '\0';
		}
		need_reset = 1;
		for( cmdid=0; cmdid<sizeof(commands)/16; cmdid++ )
			if( strcmp( commands[cmdid], cmd )==0 )
				break;
SELECT_CMD:
		switch( cmdid ){
		case CMD_TO:
		case CMD_TO2:
		{
			if( enter )
			{
				MSG("You are in a Qun, you talk to everybody.\n");
				break;
			}
			int n = atoi( arg );
			if( n < 0xFFFF )
			{
				char *p;
				p = skip_line( buddy_buf, n-1 );
				if( p )
				{
					sscanf( p, "%u%u", &n, &to_uid );
					MSG("You are talking to %s\n", to_utf8( myqq_get_buddy_name(qq, to_uid) ) );
					break;
				}
			}else{
				to_uid = n;
				MSG("You are talking to %s\n", to_utf8( myqq_get_buddy_name(qq, to_uid) ) );
				break;
			}
			MSG("to: %s is not found.\n", arg );
			break;
		}
		case CMD_SAY:
		case CMD_SAY2:
		{
			if( enter )
			{
				if( myqq_send_im_to_qun( qq, qun_int_uid, to_gb( arg ), 1 ) < 0 ){
					MSG("TIMEOUT: You message has not been sent.\n");
				}
			}else{
				if( to_uid == 0 )
				{
					MSG("say: Say to who?\n");
					break;
				}
				if( myqq_send_im_to_buddy( qq, to_uid, to_gb( arg ), 1 ) < 0 ){
					MSG("TIMEOUT: You message has not been sent.\n");
				}
			}
			break;
		}
		case CMD_EXIT:
		case CMD_EXIT2:
			qqclient_logout( qq );
			qqclient_cleanup( qq );
			MSG("Logout.\n");
			exit(0);
		case CMD_HELP:
			MSG( help_msg );
			break;
		case CMD_STATUS:
			if( strcmp( arg, "away") == 0 )
				qqclient_change_status( qq, QQ_AWAY );
			else if( strcmp( arg, "online") == 0 )
				qqclient_change_status( qq, QQ_ONLINE );
			else if( strcmp( arg, "hidden") == 0 )
				qqclient_change_status( qq, QQ_HIDDEN );
			else if( strcmp( arg, "killme") == 0 )
				qqclient_change_status( qq, QQ_KILLME );
			else if( strcmp( arg, "busy") == 0 )
				qqclient_change_status( qq, QQ_BUSY );
			else{
				MSG("Unknown status %s\n", arg );
			}
			break;
		case CMD_ENTER:
		case CMD_ENTER2:
		{
			int n = atoi( arg );
			if( n < 0xFFFF )
			{
				char *p;
				p = skip_line( qun_buf, n-1 );
				if( p )
				{
					sscanf( p, "%u%u", &n, &qun_int_uid );
					MSG("You are in Group %s\n", to_utf8( myqq_get_qun_name( qq, qun_int_uid) ) );
					enter = 1;
					break;
				}
			}else{
				qun_int_uid = n;
				MSG("You are in Group %s\n", to_utf8( myqq_get_qun_name( qq, qun_int_uid) ) );
				enter = 1;
				break;
			}
			MSG("enter: %s is not found.\n", arg );
			break;
		}
		case CMD_LEAVE:
		case CMD_LEAVE2:
			if( !enter )
			{
				MSG("Not entered.\n");
				break;
			}
			enter = 0;
			MSG("Leave %s.", to_utf8( myqq_get_qun_name( qq, qun_int_uid ) ));
			MSG("You are talking to %s\n", to_utf8( myqq_get_buddy_name( qq, to_uid ) ) );
			break;
		case CMD_QUN:
		case CMD_QUN2:
		{
			myqq_get_qun_list( qq, qun_buf, QUN_BUF_SIZE );
			MSG( to_utf8( qun_buf ) );
			break;
		}
		case CMD_UPDATE:
		case CMD_UPDATE2:
			buddy_update_list( qq );
			group_update_list( qq );
			MSG("Updating...\n");
			if( qqclient_wait( qq, 20 )<0 ){
				MSG("Timeout to update.\n");
			}
			break;
		case CMD_INFO:
		case CMD_INFO2:
		{
			if( !enter )
			{
				if( to_uid==0 )
				{
					MSG("Please choose a buddy first.\n");
				}else{
					char* buf = (char*)malloc(1024*4);	//4kb enough!
					if( myqq_get_buddy_info( qq, to_uid, buf, 1024*4 ) < 0 ){
						MSG("Fail to get infor for %s\n",
							to_utf8( myqq_get_buddy_name( qq, to_uid ) ) );
					}else{
						MSG( to_utf8( buf ) );
					}
					free(buf);
				}
			}else{
				char* buf = (char*)malloc(1024*4);	//4kb enough!
				if( myqq_get_qun_info( qq, qun_int_uid, buf, 1024*4 ) < 0 ){
					MSG("Fail to get infor for %s\n",
						to_utf8( myqq_get_qun_name( qq, qun_int_uid ) ) );
				}else{
					MSG( to_utf8( buf ) );
				}
				free(buf);
			}
			break;
		}
		case CMD_VIEW:
		case CMD_VIEW2:
			if( enter )
			{
				myqq_get_qun_member_list( qq, qun_int_uid, buddy_buf,
					BUDDY_BUF_SIZE, 0 );
				MSG( to_utf8( buddy_buf ) );
			}else{
				myqq_get_buddy_list( qq, buddy_buf, BUDDY_BUF_SIZE, 0 );
				MSG( to_utf8( buddy_buf ) );
			}
			break;
		case CMD_WHO:
		case CMD_WHO2:
			if( enter )
			{
				myqq_get_qun_member_list( qq, qun_int_uid, buddy_buf,
					QUN_BUF_SIZE, 1 );
				MSG( to_utf8( buddy_buf ) );
			}else{
				myqq_get_buddy_list( qq, buddy_buf, QUN_BUF_SIZE, 1 );
				MSG( to_utf8( buddy_buf ) );
			}
			break;
		case CMD_CHANGE:
		case CMD_CHANGE2:
			qqclient_logout( qq );
			qqclient_cleanup( qq );
			DEL( qun_buf );
			DEL( buddy_buf );
			DEL( print_buf );
			return main( 0, NULL );
			break;
		case CMD_TEST:
		//	myqq_test( qq );
			break;
		default:
			//use it as the last cmd's argument
			if( lastcmd && *input )
			{
				cmdid = lastcmd;
				strncpy( arg, input, 1008-1 );
				*input = 0;
				goto SELECT_CMD;
			}
			break;
		}
		lastcmd = cmdid;
	}
	qqclient_logout( qq );
	qqclient_cleanup( qq );
	return 0;
}
